# -*- coding: utf-8 -*-
import synth
import random
import pygame
from mingus.containers.Note import Note
import numpy

#notes = 'F C G D A E B'.split(" ")
notes = 'C D E F G A B'.split(" ")


def weighted_choice(choices, weights):
    total = sum(weights)
    r = random.uniform(0, total)
    upto = 0
    for c, w in zip(choices, weights):
        if upto + w > r:
            return c
        upto += w
    assert False, "Shouldn't get here"


def select_note(previous):
    p = notes.index(str(previous)[1])
    ps = []
    for i in range(len(notes)):
        d = abs(i - p)
        if d == 0:
            ps.append(2)
        else:
            ps.append(1. / (d * d))
    return Note(weighted_choice(notes, ps))


def sound_concat(first, second):
    fa = pygame.sndarray.samples(first)
    sa = pygame.sndarray.samples(second)
    final = numpy.concatenate((fa, sa))
    return pygame.sndarray.make_sound(final)


def sound_add(base, *rest):
    total_sounds = 1 + len(rest)
    abase = pygame.sndarray.samples(base)
    abase /= total_sounds

    for s in rest:
        o = pygame.sndarray.samples(s)
        o /= total_sounds
        abase[:len(o)] += o

MID_MARK = 10
TOP_MARK = 20


class Music(object):
    def __init__(self, framerate):
        self.framerate = framerate
        self.channels = [
            pygame.mixer.Channel(0),
            pygame.mixer.Channel(1),
            pygame.mixer.Channel(2)
            ]

        for ch in self.channels:
            ch.set_volume(0.5)

        self.instruments = [
            synth.bass(self.framerate)
            ]

        self.notes = [None] * 3

    def update(self, combo, pulse=None):
        ch = self.channels[0]

        if not ch.get_queue():
            syn = self.instruments[0]

            if self.notes[0] is None:
                note = Note(random.choice(notes))
            else:
                note = select_note(self.notes[0])
            notef = note.to_hertz() / 2

            self.notes[0] = note
            syn["attack"] = random.random() * 0.39 + 0.5
            syn['decay'] = random.random() * 0.4
            syn['sustain'] = random.random() * 0.4 + 0.4
            syn['fm_width'] = 1000
            syn['freq'] = notef / 2
            syn['fmfreq'] = notef / 2 * 4. / 5.

            if combo > MID_MARK:
                base_duration = random.choice([1, 1])
            else:
                base_duration = 1
            base_duration = float(base_duration)

            sound = syn.build(base_duration)

            parts = []

            mid = None
            if combo > MID_MARK:
                mid_parts = random.choice([1, 1, 2])
                for part in range(mid_parts):
                    syn = self.instruments[0]

                    note = Note(random.choice(notes))
                    if self.notes[1] is None:
                        note = Note(random.choice(notes))
                    else:
                        note = select_note(self.notes[1])
                    self.notes[1] = note

                    notef = note.to_hertz() / 2
                    syn["attack"] = random.random() * 0.39
                    syn['fm_width'] = 1000
                    syn['freq'] = notef
                    syn['fmfreq'] = notef * 2. / 3.

                    snd = syn.build(base_duration / mid_parts)
                    if mid is None:
                        mid = snd
                    else:
                        mid = sound_concat(mid, snd)

                    top = None
                    if combo > TOP_MARK:
                        top_parts = random.choice([3, 4, 6])
                        for part in range(top_parts):
                            syn = self.instruments[0]

                            note = Note(random.choice(notes))
                            if self.notes[2] is None:
                                note = Note(random.choice(notes))
                            else:
                                note = select_note(self.notes[2])
                            self.notes[2] = note

                            notef = note.to_hertz() * 2
                            syn["attack"] = random.random() * 0.05
                            syn['fm_width'] = 1000
                            syn['freq'] = notef
                            syn['fmfreq'] = notef * 2 / 3.
                            syn['fmdecay'] = 0.1

                            snd = syn.build(base_duration /
                                mid_parts / top_parts)
                            if top is None:
                                top = snd
                            else:
                                top = sound_concat(top, snd)
                        parts.append(top)
                    parts.append(mid)

            if parts:
                sound_add(sound, *parts)

            self.channels[0].queue(sound)


if __name__ == "__main__":
    pygame.mixer.pre_init(channels=1)
    pygame.init()
    f, _, _ = pygame.mixer.get_init()
    pygame.mixer.set_num_channels(100)
    ch = pygame.mixer.Channel(0)

    m = Music(f)

    while True:
        events = pygame.event.get()
        m.update(45)